@@ -1,23 +1,25 @@
 use crate::babelify::{Babelify, Context};
 use copyless::BoxHelper;
 use serde::{Deserialize, Serialize};
-use swc_common::Spanned;
+use swc_atoms::js_word;
+use swc_common::{BytePos, Span, Spanned};
 use swc_ecma_ast::{
     ArrayLit, ArrowExpr, AssignExpr, AwaitExpr, BinExpr, BinaryOp, BlockStmtOrExpr, CallExpr,
-    ClassExpr, CondExpr, Expr, ExprOrSpread, ExprOrSuper, FnExpr, Lit, MemberExpr, MetaPropExpr,
-    NewExpr, ObjectLit, ParenExpr, PatOrExpr, PropOrSpread, SeqExpr, SpreadElement, Super,
-    TaggedTpl, ThisExpr, Tpl, TplElement, UnaryExpr, UpdateExpr, YieldExpr,
+    Callee, ClassExpr, CondExpr, Expr, ExprOrSpread, FnExpr, Ident, Import, Lit, MemberExpr,
+    MemberProp, MetaPropExpr, MetaPropKind, NewExpr, ObjectLit, ParenExpr, PatOrExpr, PropOrSpread,
+    SeqExpr, SpreadElement, Super, SuperProp, SuperPropExpr, TaggedTpl, ThisExpr, Tpl, TplElement,
+    UnaryExpr, UpdateExpr, YieldExpr,
 };
 use swc_estree_ast::{
     flavor::Flavor, ArrayExprEl, ArrayExpression, ArrowFuncExprBody, ArrowFunctionExpression,
     AssignmentExpression, AwaitExpression, BinaryExprLeft, BinaryExpression, CallExpression,
-    Callee, ClassExpression, ConditionalExpression, Expression, FunctionExpression, LVal, Literal,
-    LogicalExpression, MemberExprProp, MemberExpression, MetaProperty, NewExpression,
-    ObjectExprProp, ObjectExpression, ObjectKey, ObjectMember, ParenthesizedExpression,
-    PrivateName, SequenceExpression, SpreadElement as BabelSpreadElement, Super as BabelSuper,
-    TaggedTemplateExprTypeParams, TaggedTemplateExpression, TemplateElVal, TemplateElement,
-    TemplateLiteral, TemplateLiteralExpr, ThisExpression, UnaryExpression, UpdateExpression,
-    YieldExpression,
+    Callee as BabelCallee, ClassExpression, ConditionalExpression, Expression, FunctionExpression,
+    Import as BabelImport, LVal, Literal, LogicalExpression, MemberExprProp, MemberExpression,
+    MetaProperty, NewExpression, ObjectExprProp, ObjectExpression, ObjectKey, ObjectMember,
+    ParenthesizedExpression, PrivateName, SequenceExpression, SpreadElement as BabelSpreadElement,
+    Super as BabelSuper, TaggedTemplateExprTypeParams, TaggedTemplateExpression, TemplateElVal,
+    TemplateElement, TemplateLiteral, TemplateLiteralExpr, ThisExpression, UnaryExpression,
+    UpdateExpression, YieldExpression,
 };
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
@@ -59,6 +61,9 @@ impl Babelify for Expr {
             Expr::Member(m) => {
                 ExprOutput::Expr(Box::alloc().init(Expression::Member(m.babelify(ctx))))
             }
+            Expr::SuperProp(m) => {
+                ExprOutput::Expr(Box::alloc().init(Expression::Member(m.babelify(ctx))))
+            }
             Expr::Cond(c) => {
                 ExprOutput::Expr(Box::alloc().init(Expression::Conditional(c.babelify(ctx))))
             }
@@ -373,16 +378,57 @@ impl Babelify for MemberExpr {
     type Output = MemberExpression;
 
     fn babelify(self, ctx: &Context) -> Self::Output {
+        let computed = self.prop.is_computed();
+
         MemberExpression {
             base: ctx.base(self.span),
-            object: Box::alloc().init(self.obj.babelify(ctx)),
+            object: Box::alloc().init(self.obj.babelify(ctx).into()),
             property: Box::alloc().init(self.prop.babelify(ctx).into()),
-            computed: self.computed,
+            computed,
             optional: Default::default(),
         }
     }
 }
 
+impl Babelify for MemberProp {
+    type Output = MemberExprProp;
+
+    fn babelify(self, ctx: &Context) -> Self::Output {
+        match self {
+            Self::Computed(c) => MemberExprProp::Expr(c.babelify(ctx).into()),
+            Self::Ident(i) => MemberExprProp::Id(i.babelify(ctx)),
+            Self::PrivateName(p) => MemberExprProp::PrivateName(p.babelify(ctx)),
+        }
+    }
+}
+
+impl Babelify for SuperPropExpr {
+    type Output = MemberExpression;
+
+    fn babelify(self, ctx: &Context) -> Self::Output {
+        let computed = self.prop.is_computed();
+
+        MemberExpression {
+            base: ctx.base(self.span),
+            object: Box::alloc().init(Expression::Super(self.obj.babelify(ctx).into())),
+            property: Box::alloc().init(self.prop.babelify(ctx).into()),
+            computed,
+            optional: Default::default(),
+        }
+    }
+}
+
+impl Babelify for SuperProp {
+    type Output = MemberExprProp;
+
+    fn babelify(self, ctx: &Context) -> Self::Output {
+        match self {
+            Self::Computed(c) => MemberExprProp::Expr(c.babelify(ctx).into()),
+            Self::Ident(i) => MemberExprProp::Id(i.babelify(ctx)),
+        }
+    }
+}
+
 impl Babelify for CondExpr {
     type Output = ConditionalExpression;
 
@@ -421,7 +467,7 @@ impl Babelify for NewExpr {
     fn babelify(self, ctx: &Context) -> Self::Output {
         NewExpression {
             base: ctx.base(self.span),
-            callee: Callee::Expr(Box::alloc().init(self.callee.babelify(ctx).into())),
+            callee: BabelCallee::Expr(Box::alloc().init(self.callee.babelify(ctx).into())),
             arguments: match self.args {
                 Some(args) => args
                     .into_iter()
@@ -493,10 +539,52 @@ impl Babelify for MetaPropExpr {
     type Output = MetaProperty;
 
     fn babelify(self, ctx: &Context) -> Self::Output {
+        let (meta, property) = match self.kind {
+            MetaPropKind::ImportMeta => (
+                Ident {
+                    span: Span {
+                        hi: self.span.hi - BytePos(5),
+                        ..self.span
+                    },
+                    sym: js_word!("import"),
+                    optional: false,
+                }
+                .babelify(ctx),
+                Ident {
+                    span: Span {
+                        lo: self.span.lo + BytePos(7),
+                        ..self.span
+                    },
+                    sym: js_word!("meta"),
+                    optional: false,
+                }
+                .babelify(ctx),
+            ),
+            MetaPropKind::NewTarget => (
+                Ident {
+                    span: Span {
+                        hi: self.span.hi - BytePos(7),
+                        ..self.span
+                    },
+                    sym: js_word!("new"),
+                    optional: false,
+                }
+                .babelify(ctx),
+                Ident {
+                    span: Span {
+                        hi: self.span.hi + BytePos(4),
+                        ..self.span
+                    },
+                    sym: js_word!("target"),
+                    optional: false,
+                }
+                .babelify(ctx),
+            ),
+        };
         MetaProperty {
             base: ctx.base(self.span()),
-            meta: self.meta.babelify(ctx),
-            property: self.prop.babelify(ctx),
+            meta,
+            property,
         }
     }
 }
@@ -569,13 +657,14 @@ impl Babelify for ParenExpr {
     }
 }
 
-impl Babelify for ExprOrSuper {
+impl Babelify for Callee {
     type Output = Expression;
 
     fn babelify(self, ctx: &Context) -> Self::Output {
         match self {
-            ExprOrSuper::Expr(e) => e.babelify(ctx).into(),
-            ExprOrSuper::Super(s) => Expression::Super(s.babelify(ctx)),
+            Callee::Expr(e) => e.babelify(ctx).into(),
+            Callee::Super(s) => Expression::Super(s.babelify(ctx)),
+            Callee::Import(i) => Expression::Import(i.babelify(ctx)),
         }
     }
 }
@@ -590,6 +679,16 @@ impl Babelify for Super {
     }
 }
 
+impl Babelify for Import {
+    type Output = BabelImport;
+
+    fn babelify(self, ctx: &Context) -> Self::Output {
+        BabelImport {
+            base: ctx.base(self.span),
+        }
+    }
+}
+
 impl Babelify for ExprOrSpread {
     type Output = ArrayExprEl;
 
